package redner

import (
	"bytes"
	"context"
	"errors"
	"fmt"

	"github.com/rs/zerolog/log"
	"gopkg.in/gomail.v2"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/rendering"
)

var DefaultResetMailTemplate = models.Template{
	Language: "mustache",
	Produces: "text/plain",
	Metadata: models.Metadata{
		"To":      "{{user.firstname}} {{user.lastname}} <{{user.email}}>",
		"Subject": "Password Reset",
	},
	BodyFormat: "text",
	Body: `You have requested a password reset for account {{user.username}}

You can do so by visiting the following link:

	{{reset-password-url}}

If you haven't requested a password reset, just ignore this message. No harm is done.

Thanks!
`,
}

var DefaultSubscribeTempate = models.Template{
	Language: "mustache",
	Produces: "text/plain",
	Metadata: models.Metadata{
		"To":      "{{user.firstname}} {{user.lastname}} <{{user.email}}>",
		"Subject": "Welcome to Rednerd!",
	},
	BodyFormat: "text",
	Body: `Welcome to Redner!

You have requested a redner account "{{user.username}}".

You can set your password by visiting the following link:

	{{reset-password-url}}

If you haven't requested a redner account reset, just ignore this message. No harm is done.

Thanks!
`,
}

func isEmailHeader(key string) bool {
	switch key {
	case "Subject", "To":
		return true
	default:
		return false
	}
}

func SendMail(
	ctx context.Context,
	db database.SQLHelper,
	templateEngineRegistry *rendering.TemplateEngineRegistry,
	engineRegistry *rendering.EngineRegistry,
	template models.Template, record models.Record,
) error {
	targetType := template.Produces
	if targetType == "text/mjml" {
		targetType = "text/html"
	}

	if targetType != "text/plain" && targetType != "text/html" {
		return fmt.Errorf("template produces a type unsuitable for emails: %s", targetType)
	}

	template.Metadata.Add("to", "{{user.email}}")

	pipeline, err := templateEngineRegistry.NewPipeline(
		ctx, engineRegistry, &template, nil, targetType,
	)
	if err != nil {
		return err
	}

	var doc models.Document
	w := rendering.NewDocumentWriter(&doc)
	if err := pipeline.Render(ctx, w, record); err != nil {
		return err
	}

	b, err := doc.Bytes()
	if err != nil {
		return errors.New("could not read rendered email body")
	}
	message := gomail.NewMessage()
	message.SetBody(doc.Type, string(b))
	for k, v := range doc.Metadata {
		if isEmailHeader(k) {
			message.SetHeader(k, v)
		}
	}

	var buf bytes.Buffer

	if _, err := message.WriteTo(&buf); err != nil {
		return err
	}
	if err := models.QueueMail(db, "", []string{doc.Metadata["to"]}, buf.Bytes()); err != nil {
		return err
	}

	return nil
}

func ResetPassword(
	ctx context.Context,
	db database.SQLHelper,
	templateEngineRegistry *rendering.TemplateEngineRegistry,
	engineRegistry *rendering.EngineRegistry,
	baseURL string,
	user models.UserDB, template *models.Template,
) error {
	if user.Email == "" {
		return fmt.Errorf("user %s has no email", user.Username)
	}
	if template == nil {
		template = &DefaultResetMailTemplate
	}
	token, err := models.NewResetPasswordToken(db, user.Username)
	if err != nil {
		return err
	}
	log.Info().Str("url", "/reset-password/"+token).
		Str("email", user.Email).
		Msg("send an email with the reset url")

	if err := SendMail(ctx, db, templateEngineRegistry, engineRegistry, *template, map[string]interface{}{
		"user": map[string]interface{}{
			"firstname": user.Firstname,
			"lastname":  user.Lastname,
			"email":     user.Email,
			"username":  user.Username,
		},
		"reset-password-url": baseURL + "/reset-password/" + token,
	}); err != nil {
		return err
	}

	return nil
}
