package rendering

import (
	"context"
	"errors"
	"fmt"
	"runtime/debug"
	"sync"
	"sync/atomic"

	"github.com/rs/zerolog"
	"github.com/rs/zerolog/log"
)

var (
	ErrEngineShuttingDown = errors.New("engine shutting down")

	// ErrEngineCrashed must be returned by the underlying engine when the
	// engine cannot be used anymore.
	ErrEngineCrashed = errors.New("engine stopped")
)

type EngineFactory func(zerolog.Logger) (Engine, error)

func NewPooledEngine(factory EngineFactory, size int, log zerolog.Logger) (*PooledEngine, error) {
	if size <= 0 {
		return nil, errors.New("PooledEngine size must be a positive number")
	}
	e := PooledEngine{
		factory: factory,
		jobs:    make(chan *PooledEngineJob),
		crashed: make(chan error),
		log:     log,
	}
	e.wg.Add(1)
	go e.crashHandler()
	log.Debug().Int("worker-pool-size", size).Msg("spawning workers")
	for i := 0; i != size; i++ {
		if err := e.spawnWorker(); err != nil {
			e.Close()

			return nil, err
		}
	}
	log.Info().Int("worker-pool-size", size).Msg("successfully spawned workers")

	return &e, nil
}

type PooledEngineJob struct {
	ctx    context.Context
	w      DocumentWriter
	r      DocumentReader
	toType string
	result chan error
}

type PooledEngine struct {
	factory     EngineFactory
	info        EngineInfo
	workerCount uint64
	workerSeq   uint64
	jobs        chan *PooledEngineJob
	crashed     chan error
	wg          sync.WaitGroup
	log         zerolog.Logger
}

func (e *PooledEngine) Info() EngineInfo {
	return e.info
}

func (e *PooledEngine) Render(
	ctx context.Context, w DocumentWriter, r DocumentReader, toType string,
) error {
	job := PooledEngineJob{
		ctx:    ctx,
		w:      w,
		r:      r,
		toType: toType,
		result: make(chan error),
	}
	select {
	case <-ctx.Done():
		return ctx.Err()
	case e.jobs <- &job:
	}

	select {
	case <-ctx.Done():
		// we still need to consume the result
		go func() { <-job.result }()

		return ctx.Err()
	case err := <-job.result:
		return err
	}
}

func (e *PooledEngine) Close() {
	close(e.jobs)
	for job := range e.jobs {
		job.result <- ErrEngineShuttingDown
	}
	close(e.crashed)
	e.wg.Wait()
}

func (e *PooledEngine) spawnWorker() error {
	id := e.workerSeq
	e.workerSeq++
	engineLog := log.With().Uint64("engine-worker-id", id).Logger()
	engine, err := e.factory(engineLog)
	if err != nil {
		return fmt.Errorf("failed to spawn worker: %w", err)
	}
	if id == 0 {
		e.info = engine.Info()
	}
	e.wg.Add(1)
	atomic.AddUint64(&e.workerCount, 1)
	go e.worker(engine, engineLog)

	return nil
}

func (e *PooledEngine) worker(engine Engine, log zerolog.Logger) {
	defer e.wg.Done()
	defer func() {
		if r := recover(); r != nil {
			stack := debug.Stack()
			err, ok := r.(error)
			if ok {
				err = fmt.Errorf("panic: %w", err)
			} else {
				err = fmt.Errorf("panic: %v", r)
			}
			log.Err(err).Str("stack", string(stack)).Msg("engine stopped (panic)")
			e.crashed <- err
		}
	}()

	for job := range e.jobs {
		err := engine.Render(job.ctx, job.w, job.r, job.toType)
		job.result <- err
		if errors.Is(err, ErrEngineCrashed) {
			log.Err(err).Msg("engine stopped")
			e.crashed <- err

			return
		}
	}
}

func (e *PooledEngine) crashHandler() {
	defer e.wg.Done()
	for range e.crashed {
		atomic.AddUint64(&e.workerCount, ^uint64(0))
		e.log.Warn().Msg("an engine stopped, spawning a new worker")
		if err := e.spawnWorker(); err != nil {
			e.log.Fatal().Err(err).Msg("could not spawn a new worker")
		}
	}
}
