package handlers

import (
	"encoding/json"
	"errors"
	"fmt"
	"net/http"

	"github.com/go-openapi/runtime"
	"github.com/go-openapi/runtime/middleware"
	"github.com/rs/zerolog"

	"orus.io/orus-io/rednerd/models"
)

// ErrInvalid is defined to be matched against in handlers.
var ErrInvalid = errors.New("invalid")

// ErrForbidden is defined to be matched against in handlers.
var ErrForbidden = errors.New("forbidden")

// ErrNotFound is defined to be matched inside the functional handlers
// to avoid returning errors to the client.
var ErrNotFound = errors.New("not found")

// ErrBadRequest is defined to be matched inside the functional handlers.
var ErrBadRequest = errors.New("bad request")

func NotFound(msg string) error {
	return fmt.Errorf("%w: %s", ErrNotFound, msg)
}

func BadRequest(msg string) error {
	return fmt.Errorf("%w: %s", ErrBadRequest, msg)
}

func HTTPNotFound(msg string) middleware.Responder {
	return NewErrorResponder(models.NewError(http.StatusNotFound, msg))
}

func HTTPForbidden(msg string) middleware.Responder {
	return NewErrorResponder(models.NewError(http.StatusForbidden, msg))
}

func HTTPBadRequest(msg string) middleware.Responder {
	return NewErrorResponder(models.NewError(http.StatusBadRequest, msg))
}

// HandleStandardError returns a responder for a given error, or nil if the error
// is not a known one.
func HandleStandardError(err error, log *zerolog.Logger) middleware.Responder {
	switch {
	case errors.Is(err, ErrNotFound):
		log.Warn().Err(err).Msg("not found")

		return HTTPNotFound(err.Error())
	case errors.Is(err, ErrInvalid):
		log.Warn().Err(err).Msg("invalid")

		return HTTPBadRequest(err.Error())
	case errors.Is(err, ErrBadRequest):
		log.Warn().Err(err).Msg("bad request")

		return HTTPBadRequest(err.Error())
	case errors.Is(err, ErrForbidden):
		log.Warn().Err(err).Msg("forbidden")

		return HTTPForbidden(err.Error())
	default:
		return nil
	}
}

func HandleError(err error, log *zerolog.Logger, customHandlers ...func(err error) middleware.Responder) middleware.Responder {
	for _, h := range customHandlers {
		if r := h(err); r != nil {
			return r
		}
	}
	if r := HandleStandardError(err, log); r != nil {
		return r
	}

	return InternalError(err, log, false)
}

// ErrorResponder is a generic http error responder.
type ErrorResponder struct { //nolint:errname
	code int
	err  *models.Error
}

func NewErrorResponder(err *models.Error) *ErrorResponder {
	return &ErrorResponder{
		code: int(err.Code),
		err:  err,
	}
}

func (r ErrorResponder) Error() string {
	return "BadRequest" + r.err.Error()
}

func (r ErrorResponder) Unwrap() error {
	return r.err
}

// WriteResponse to the client.
func (r ErrorResponder) WriteResponse(rw http.ResponseWriter, producer runtime.Producer) {
	rw.WriteHeader(r.code)
	if r.err != nil {
		enc := json.NewEncoder(rw)
		enc.SetEscapeHTML(false)
		if err := enc.Encode(models.FromError(r.err)); err != nil {
			panic(err) // let the recovery middleware deal with this
		}
	}
}

// InternalErrorResponder ...
// InternalErrorResponder is a helper to filter out the error content sent
// to the client this filtering is dependent on the global debug flag.
// ie: if debug is set then the client will receive the detail
// of the internal error.
// This is something you obviously do NOT want to do in production mode.
type InternalErrorResponder struct {
	err   error
	log   *zerolog.Logger
	debug bool
}

// InternalError is a helper function to log and internal error and obtain
// a Responder that you can use as a return value in your handler.
func InternalError(err error, log *zerolog.Logger, debug bool) *InternalErrorResponder {
	log.Err(err).Msg("internal error")

	return &InternalErrorResponder{err: err, log: log, debug: debug}
}

// WriteResponse to the client.
func (r *InternalErrorResponder) WriteResponse(rw http.ResponseWriter, producer runtime.Producer) {
	rw.WriteHeader(http.StatusInternalServerError)
	if r.err != nil && r.debug {
		// clients are allowed to see internal errors content only if the
		// global debug mode is activated on the ALPE api
		if err := producer.Produce(rw, models.FromError(r.err)); err != nil {
			panic(err) // let the recovery middleware deal with this
		}
	}
}
