package handlers

import (
	"context"
	"errors"

	"github.com/go-openapi/runtime/middleware"
	"github.com/rs/zerolog"

	redner "orus.io/orus-io/rednerd/lib"
	"orus.io/orus-io/rednerd/models"
	op "orus.io/orus-io/rednerd/restapi/operations/rendering"
)

// NewRenderHandler creates a RenderHandler.
func NewRenderHandler(renderer *redner.Renderer) *RenderHandler {
	return &RenderHandler{renderer}
}

// RenderHandler handles POST /api/v1/render.
type RenderHandler struct {
	renderer *redner.Renderer
}

func (h *RenderHandler) handle(
	ctx context.Context,
	principal *models.Principal,
	template *models.Template,
	data models.Dataset,
	document *models.Document,
	metadata models.Metadata,
	toType string,
) ([]*models.Document, error) {
	if template == nil && document == nil {
		return nil, models.BadRequest(
			"no template nor document. Must have one")
	}
	documents, err := h.renderer.Render(
		ctx, principal.Subject, template, data, document, metadata, toType,
		principal.EnabledRequestLogging,
	)
	if errors.Is(err, redner.ErrInvalidInput) {
		return nil, BadRequest(err.Error())
	} else if err != nil {
		return nil, err
	}

	return documents, nil
}

// Handle the incoming request.
func (h *RenderHandler) Handle(params op.RenderParams, principal *models.Principal) middleware.Responder {
	accept := params.Request.Accept
	ctx := params.HTTPRequest.Context()
	// TODO if accept is empty, get a 'accept' from the http header

	// TODO checks if the user has render access to the template

	docs, err := h.handle(
		ctx,
		principal,
		params.Request.Template,
		params.Request.Data,
		params.Request.Document,
		params.Request.Metadata,
		string(accept),
	)
	if err != nil {
		if params.HTTPRequest.Context().Err() != nil {
			return op.NewRenderDefault(499)
		}

		return HandleError(err, zerolog.Ctx(ctx))
	}

	return op.NewRenderOK().WithPayload(docs)
}
