package handlers

import (
	"context"
	"errors"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/rendering"
	op "orus.io/orus-io/rednerd/restapi/operations/template_management"
)

// NewTemplateVarlistHandler creates a TemplateVarlistHandler.
func NewTemplateVarlistHandler(
	db *sqlx.DB,
	templateRegistry *rendering.TemplateEngineRegistry,
) *TemplateVarlistHandler {
	return &TemplateVarlistHandler{db, templateRegistry}
}

// TemplateVarlistHandler handles POST /api/v1/render.
type TemplateVarlistHandler struct {
	db               *sqlx.DB
	templateRegistry *rendering.TemplateEngineRegistry
}

func (h *TemplateVarlistHandler) handle(
	ctx context.Context,
	template *models.Template,
) ([]string, error) {
	if template == nil {
		return nil, BadRequest("no template given")
	}

	log := zerolog.Ctx(ctx)

	if template.Account != "" && template.Name != "" {
		templatedb := models.TemplateDB{}
		sql := database.NewSQLHelper(ctx, h.db, *log)
		if err := sql.GetWhere(
			&templatedb,
			squirrel.Eq{
				models.TemplateDBAccountColumn: template.Account,
				models.TemplateDBNameColumn:    template.Name,
			},
		); err != nil {
			return nil, err
		}
		templatedb.FromDB()
		template = &templatedb.Template
	}
	tmpl, err := h.templateRegistry.Load(ctx, template)
	if err != nil {
		if errors.Is(err, rendering.ErrUnknownTemplateLanguage) {
			return nil, BadRequest(
				"unknown template language: " + template.Language,
			)
		}

		return nil, BadRequest(
			"error loading template: " + err.Error(),
		)
	}
	varlist, err := tmpl.Varlist()
	if err != nil {
		return nil, BadRequest(
			"error listing template variables: " + err.Error(),
		)
	}

	return varlist, nil
}

// Handle the incoming request.
func (h *TemplateVarlistHandler) Handle(params op.TemplateVarlistParams, principals *models.Principal) middleware.Responder {
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	varlist, err := h.handle(
		ctx,
		params.Request,
	)
	if err != nil {
		return HandleError(err, log)
	}

	return op.NewTemplateVarlistOK().WithPayload(varlist)
}
