package handlers

import (
	"context"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// ErrNoSuchUserApikey is returned if the user does not exist.
var ErrNoSuchUserApikey = NotFound("no such user")

// NewUserApikeyDeleteHandler returns a UserApikeyDeleteHandler.
func NewUserApikeyDeleteHandler(db *sqlx.DB) *UserApikeyDeleteHandler {
	return &UserApikeyDeleteHandler{
		db: db,
	}
}

// UserApikeyDeleteHandler handles GET /user.
type UserApikeyDeleteHandler struct {
	db *sqlx.DB
}

func (h *UserApikeyDeleteHandler) handle(
	ctx context.Context, username string, name string,
) error {
	log := zerolog.Ctx(ctx)
	tx, err := database.Begin(ctx, h.db)
	if err != nil {
		return err
	}
	defer tx.RollbackIfOpened(*log)
	r, err := database.ExecContext(ctx, tx,
		squirrel.Delete(models.ApikeyDBTable).
			From(models.ApikeyDBTable).
			Where(squirrel.Eq{
				models.ApikeyDBUsernameColumn: username,
				models.ApikeyNameColumn:       name,
			}),
		log,
	)
	if err != nil {
		return err
	}
	count, err := r.RowsAffected()
	if err != nil {
		return err
	}
	if count == 0 {
		return ErrNoSuchUserApikey
	}

	return tx.Commit()
}

// Handle handles the incoming request.
func (h *UserApikeyDeleteHandler) Handle(
	params user_management.UserApikeyDeleteParams, principal *models.Principal,
) middleware.Responder {
	if !principal.IsAdmin && principal.Subject != params.Username {
		return user_management.NewUserApikeyDeleteUnauthorized().
			WithPayload(models.Unauthorized(
				"only admins can delete apikey that they don't own",
			))
	}
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	err := h.handle(ctx, params.Username, params.Name)
	if err != nil {
		return HandleError(err, log)
	}

	return user_management.NewUserApikeyDeleteNoContent()
}
