package handlers

import (
	"context"
	"net/http"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// ErrNoSuchUser is returned if the user does not exist.
var ErrNoSuchUser = NotFound("no such user")

// NewUserDeleteHandler returns a UserDeleteHandler.
func NewUserDeleteHandler(db *sqlx.DB) *UserDeleteHandler {
	return &UserDeleteHandler{
		db: db,
	}
}

// UserDeleteHandler handles GET /user.
type UserDeleteHandler struct {
	db *sqlx.DB
}

func (h *UserDeleteHandler) handle(
	ctx context.Context, username string,
) error {
	log := zerolog.Ctx(ctx)
	tx, err := database.Begin(ctx, h.db)
	if err != nil {
		return err
	}
	defer tx.RollbackIfOpened(*log)

	r, err := database.ExecContext(ctx, tx,
		squirrel.Delete(models.UserDBTable).
			From(models.UserDBTable).
			Where(squirrel.Eq{models.UserinfoUsernameColumn: username}),
		log,
	)
	if err != nil {
		return err
	}
	count, err := r.RowsAffected()
	if err != nil {
		return err
	}
	if count == 0 {
		return ErrNoSuchUser
	}
	if _, err := database.ExecContext(ctx, tx,
		squirrel.Delete(models.AccountDBTable).
			Where(squirrel.Eq{models.AccountDBNameColumn: username}),
		log,
	); err != nil {
		return err
	}

	return tx.Commit()
}

// Handle handles the incoming request.
func (h *UserDeleteHandler) Handle(
	params user_management.UserDeleteParams, principal *models.Principal,
) middleware.Responder {
	if !principal.IsAdmin && principal.Subject != params.Username {
		return user_management.NewUserApikeyAddUnauthorized().
			WithPayload(models.Unauthorized(
				"only admins can delete users, except deleting you own account",
			))
	}
	if principal.Subject == params.Username && principal.IsAdmin {
		return user_management.NewUserDeleteDefault(http.StatusBadRequest).
			WithPayload(models.BadRequest(
				"An admin account cannot delete its own account",
			))
	}
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	err := h.handle(ctx, params.Username)
	if err != nil {
		return HandleError(err, log)
	}

	return user_management.NewUserDeleteNoContent()
}
