package apitester

import (
	"io"
	"net/http"

	"github.com/k3a/html2text"
	"github.com/stretchr/testify/require"

	"orus.io/orus-io/rednerd/models"
)

// Login authenticates the user for the subsequent requests.
func (tester *APITester) Login(username, password string) {
	apitest := tester.APITest("login")
	if tester.debug {
		apitest = apitest.Debug()
	}
	creds := models.Credentials{
		Username: username,
		Password: password,
	}
	var token string
	r := apitest.
		Post("/api/v1/auth/authenticate").
		JSON(&creds).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	r.JSON(&token)
	tester.SetHeader("Authorization", "Bearer "+token)
}

// Logout resets the "Authorization" headers.
func (tester *APITester) Logout() {
	tester.SetHeader("Authorization", "")
}

// GetUserInfo returns the connected user information.
func (tester *APITester) GetUserInfo() models.Userinfo {
	apitest := tester.APITest("login")
	if tester.debug {
		apitest = apitest.Debug()
	}
	var user models.Userinfo
	r := apitest.
		Get("/api/v1/auth/userinfo").
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	r.JSON(&user)

	return user
}

// CreateUser creates a new user and sets its password.
func (tester *APITester) CreateUser(username, password string) {
	apitest := tester.APITest("user_create")
	if tester.debug {
		apitest = apitest.Debug()
	}
	user := models.Userinfo{
		Username: username,
	}
	r := apitest.
		Post("/api/v1/user").
		JSON(&user).
		Expect(tester.t).
		Status(http.StatusCreated).
		End()
	var info models.Userinfo
	r.JSON(&info)
	require.Equal(tester.t, username, info.Username)
	require.False(tester.t, info.IsAdmin)

	// set password.
	apitest = tester.APITest("password_set")
	if tester.debug {
		apitest = apitest.Debug()
	}
	_ = apitest.
		Post("/api/v1/user/" + username + "/password").
		JSON(&JSONObj{"password": password}).
		Expect(tester.t).
		Status(http.StatusNoContent).
		End()
}

// CreateTemplate creates a new template associated to an Account.
func (tester *APITester) CreateTemplate(
	username string,
	body, response *models.Template,
) {
	apitest := tester.APITest("template_create")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Post("/api/v1/template/" + username).
		JSON(body).
		Expect(tester.t).
		Status(http.StatusCreated).
		End()
	r.JSON(response)
}

// Render renders some data and returns a document.
func (tester *APITester) Render(
	data *models.RenderRequest,
	result *[]*models.Document,
) {
	apitest := tester.APITest("render")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Post("/api/v1/render/").
		JSON(data).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	r.JSON(result)
}

// Renderlogs gives us a csv document containing the
// Rendering data for invoicing.
func (tester *APITester) RenderLogs(
	period string,
) string {
	apitest := tester.APITest("renderlogs " + period)
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Get("/api/v1/renderlogs/" + period).
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	b, err := io.ReadAll(r.Response.Body)
	require.NoError(tester.t, err)

	return string(b)
}

func (tester *APITester) ListEngines() string {
	apitest := tester.APITest("list_engines")
	if tester.debug {
		apitest = apitest.Debug()
	}
	r := apitest.
		Get("/docs/engines").
		Expect(tester.t).
		Status(http.StatusOK).
		End()
	b, err := io.ReadAll(r.Response.Body)
	require.NoError(tester.t, err)
	engines := html2text.HTML2Text(string(b))

	return engines
}
